import React, { useState, useEffect, useMemo, useContext } from 'react'
import { Input, message, ConfigProvider } from 'antd'
import Icon, { PlusOutlined } from '@ant-design/icons'
import * as icons from '@ant-design/icons'
import { GetFieldValueFromTable, getUserName, getGroupName } from '../../../../apis/process'
import styles from './index.less'

const iconList = Object.keys(icons).filter((item) => typeof icons[item] === 'object')
const initUserInfo = {
  fullName: '【本人姓名】',
  Phone: '【本人电话】',
  depart: { name: '【本人部门】' }
}
const loaclPaths = ['【本人姓名】', '【本人电话】', '【本人部门】']

const debounce = (fn) => {
  let t = null
  return function (e) {
    const context = this
    const args = arguments
    if (t) {
      clearTimeout(t)
    }
    t = setTimeout(function () {
      fn.call(context, args)
    }, 200)
  }
}

const TextInput = (props) => {

  const preview = sessionStorage.getItem('FormRender')
  const userInfo = preview === 'preview' ? initUserInfo : window?.globalConfig?.userInfo || initUserInfo
  const { value, onChange, schema, addons } = props
  const [showValue, setShowValue] = useState(value)
  const { title, disabled, placeholder, presetValue, addonBefore, addonAfter, maxLength, rules, uniqueVerify, tableName, fieldName, isStoreID, textDefalut } = schema

  const handleChange = (e) => {
    if (addons) {
      onChange(e.target.value)
      setShowValue(e.target.value)
    }
  }

  //仅仅支持本人姓名
  const getUserInfo = async (userID) => {
    const { code, data } = await getUserName(userID)
    if (code === 0) {
      if (Array.isArray(data) && data.length > 0) {
        setShowValue(data[0])
      }
    }
  }

  const getDeptInfo = async (groupIds) => {
    const { code, data } = await getGroupName(groupIds)
    if (code === 0) {
      if (Array.isArray(data) && data.length > 0) {
        setShowValue(data[0])
      }
    }
  }

  useEffect(() => {
    let valueNext = presetValue
    if (preview !== 'preview') {
      if (!value) {
        if (textDefalut === '【本人姓名】') {
          if (isStoreID) {
            valueNext = userInfo.OID + ''
          } else {
            valueNext = userInfo.fullName
          }
        } else if (textDefalut === '【本人部门】') {
          if (isStoreID) {
            valueNext = userInfo.depart.OID + ''
          } else {
            valueNext = userInfo.depart.name
          }
        } else if (textDefalut === '【本人电话】') {
          valueNext = userInfo.Phone
        }
      }
    }
    if (addons) {
      addons.setValue(addons.dataPath, valueNext || '')
    } else {
      onChange(valueNext || '')
    }
    setShowValue(valueNext)
    if (isStoreID && valueNext) {
      if (textDefalut === '【本人姓名】') {
        getUserInfo(valueNext)
      } else if (textDefalut === '【本人部门】') {
        getDeptInfo(valueNext)
      }
    }
  }, [])

  useEffect(() => {
    if (uniqueVerify === '表名/字段名' && tableName && fieldName) {
      if (addons) {
        let getValue = async (params) => {
          const [rule, value, callback] = params
          const { code, data, msg } = await GetFieldValueFromTable(tableName, fieldName, `${fieldName}=${value}`)
          if (Array.isArray(data) && data.length) {
            callback(new Error(`${title}已重复，请重新输入`));
          } else {
            callback()
          }
        }
        let uniqueRules = [
          {
            validator: debounce(getValue)
          }
        ]
        addons.setSchemaByPath(addons.dataPath, { ...schema, rules: uniqueRules })
      }
    }
  }, [uniqueVerify, tableName, fieldName])

  return (
    <div className={styles.textInput} isdisabled={JSON.stringify(disabled)}>
      <Input
        style={{ color: loaclPaths.includes(presetValue) ? 'blue' : '' }}
        disabled={disabled}
        value={showValue}
        maxLength={maxLength}
        placeholder={disabled ? (placeholder || '') : (placeholder || '请输入内容')}
        onChange={handleChange}
        addonBefore={iconList.includes(addonBefore) ? <Icon style={{ color: 'pink' }} component={icons[addonBefore]} /> : addonBefore}
        addonAfter={iconList.includes(addonAfter) ? <Icon component={icons[addonAfter]} /> : addonAfter}
      />
    </div>
  )

}

export default TextInput
